/*************************************************************
* Raffiche un document XML pris en entre
* Utilise un parser de type SAX
*************************************************************/


import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;


public class Sax extends DefaultHandler
{
 static private Writer  out;


 /************************************************************
 * Quelques mthodes simplifiant la tche...
 ************************************************************/

 // Permet de n'avoir que des exceptions de type SAXException
 private void emit(String s) throws SAXException
 {
   try
   {
     out.write(s);
     out.flush();
   }
   catch (IOException e)
   {
     throw new SAXException("I/O error", e);
   }
 }


 // Saute une ligne
 private void nl() throws SAXException
 {
   String lineEnd =  System.getProperty("line.separator");
   try
   {
     out.write(lineEnd);
   }
   catch (IOException e)
   {
     throw new SAXException("I/O error", e);
   }
 }


 /************************************************************
 * Dfinition des mthodes SAX
 * Voir org.xml.sax.helpers.DefaultHandler
 ************************************************************/
 // Dbut de la lecture du document
 public void startDocument() throws SAXException
 {
   nl();
 }


 // Fin de la lecture du document
 public void endDocument() throws SAXException
 {
   try
   {
     nl();
     out.flush();
   }
   catch (IOException e)
   {
     throw new SAXException("I/O error", e);
   }
 }


 // Dbut d'une balise
 public void startElement(String namespaceURI, String lName, String qName,
                          Attributes attrs) throws SAXException
 {
   String eName = lName;  // nom de l'lment
   if ("".equals(eName))
     eName = qName;  // namespaceAware = false

   emit("\nBalise " + eName);

   // Lecture des attributs de la balise
   if (attrs != null)
   {
     for (int i = 0; i < attrs.getLength(); i++)
     {
       String aName = attrs.getLocalName(i);  // Nom de l'attribut
       if ("".equals(aName))
         aName = attrs.getQName(i);
 
       emit("  ");
       emit("\n    attribut de " + eName + " : " + aName + " valeur : " + attrs.getValue(i));
     }
   }
 }


 // Fin de la balise
 public void endElement(String namespaceURI, String sName, String qName)
             throws SAXException
 {
   emit(" ");
 }


 // Squence de caractres dans une balise
 public void characters(char buf[], int offset, int len) throws SAXException
 {
   String s = new String(buf, offset, len);
   emit(" : " + s);
 }



 // Programme principal
 public static void main(String argv[])
 {
   if (argv.length != 1)
   {
     System.err.println("Usage : cmd nom_de_fichier.xml");
     System.exit(1);
   }
        
   // Nouveau manipulateur SAX
   DefaultHandler handler = new Sax();

   // Utilise le parser SAX par dfaut (non validant)
   SAXParserFactory factory = SAXParserFactory.newInstance();
   try
   {
     // Initialisation du flux de sortie
     out = new OutputStreamWriter(System.out, "UTF8");

     // Parcours du fichier d'entre
     SAXParser saxParser = factory.newSAXParser();
     saxParser.parse( new File(argv[0]), handler);
   }
   catch (Throwable t)
   {
     t.printStackTrace();
   }
   System.exit(0);
 }
}
